'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _possibleConstructorReturn = _interopDefault(require('@babel/runtime/helpers/possibleConstructorReturn'));
var _getPrototypeOf = _interopDefault(require('@babel/runtime/helpers/getPrototypeOf'));
var _inherits = _interopDefault(require('@babel/runtime/helpers/inherits'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var React = _interopDefault(require('react'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var _assertThisInitialized = _interopDefault(require('@babel/runtime/helpers/assertThisInitialized'));
var PropTypes = _interopDefault(require('prop-types'));
var ReactDOM = _interopDefault(require('react-dom'));
var server = require('react-dom/server');
var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));

var canUseDom = (function () {
  return typeof window !== 'undefined';
});

var capitalize = (function (word) {
  return word.replace(/^./, function (letter) {
    return letter.toUpperCase();
  });
});

var hasOwnProperty = (function (object, prop) {
  return Object.prototype.hasOwnProperty.call(object, prop);
});

var HAS_RESOLVED = 'ReactResolver.HAS_RESOLVED';
var IS_CLIENT = 'ReactResolver.IS_CLIENT';

var Resolver =
/*#__PURE__*/
function (_React$Component) {
  _inherits(Resolver, _React$Component);

  function Resolver(props, context) {
    var _this;

    _classCallCheck(this, Resolver);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Resolver).call(this, props, context)); // Internal tracking variables

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "unMounted", true);

    _this[HAS_RESOLVED] = false;
    _this[IS_CLIENT] = false;
    _this.state = _this.computeState(_this.props, {
      pending: {},
      resolved: {}
    });

    if (_this.isPending(_this.state)) {
      _this.resolve(_this.state);

      _this[HAS_RESOLVED] = false;
    } else {
      _this[HAS_RESOLVED] = true;
    }

    return _this;
  }

  _createClass(Resolver, [{
    key: "cached",
    value: function cached(resolve) {
      var _this$props = this.props,
          props = _this$props.props,
          cache = _this$props.cache,
          cacheValidator = _this$props.cacheValidator;
      var cached = null;

      if (cache || !canUseDom()) {
        if (hasOwnProperty(props, resolve)) {
          cached = props[resolve];
        } else if (this.context.resolver) {
          cached = this.context.resolver.cached(resolve);
        }
      }

      if (cached && cacheValidator && typeof cacheValidator === 'function') {
        return cacheValidator(props);
      }

      return cached;
    }
  }, {
    key: "componentWillMount",
    value: function componentWillMount() {
      this.unMounted = false;
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this[IS_CLIENT] = true;
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.unMounted = true;
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      var _this$computeState = this.computeState(nextProps, this.state),
          pending = _this$computeState.pending,
          resolved = _this$computeState.resolved; // Next state will resolve async props again, but update existing sync props


      var nextState = {
        pending: pending,
        resolved: resolved
      };
      this.setAtomicState(nextState);
    }
  }, {
    key: "computeState",
    value: function computeState(thisProps, state) {
      var _this2 = this;

      var resolve = thisProps.resolve;
      var nextState = state;
      Object.keys(resolve).forEach(function (name) {
        var cached = _this2.cached(name);

        if (!hasOwnProperty(state.resolved, name) && !hasOwnProperty(state.pending, name) && !_this2.isValidCache(cached)) {
          nextState.pending[name] = resolve[name];
        } else if (cached) {
          nextState.resolved[name] = true;
        }
      });
      return nextState;
    }
  }, {
    key: "getChildContext",
    value: function getChildContext() {
      return {
        resolver: this
      };
    }
  }, {
    key: "isValidCache",
    value: function isValidCache(cache) {
      return cache !== null && typeof cache !== 'undefined' && (_typeof(cache) !== 'object' || Object.keys(cache).length > 0) && (!Array.isArray(cache) || cache.length > 0);
    }
  }, {
    key: "isPending",
    value: function isPending() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.state;
      return Object.keys(state.pending).length > 0;
    }
  }, {
    key: "isParentPending",
    value: function isParentPending() {
      var resolver = this.context.resolver;

      if (resolver) {
        return resolver.isPending() || resolver.isParentPending();
      }

      return false;
    }
  }, {
    key: "onResolve",
    value: function onResolve(state) {
      if (this.props.onResolve) {
        return this.props.onResolve(state);
      } else if (this.context.resolver) {
        return this.context.resolver.onResolve(state);
      } else {
        return state;
      }
    }
  }, {
    key: "render",
    value: function render() {
      // Avoid rendering until ready
      if (!this[HAS_RESOLVED]) {
        return false;
      } // If render is called again (e.g. hot-reloading), re-resolve


      if (this.isPending(this.state)) {
        this.resolve(this.state);
      } // Both those props provided by parent & dynamically resolved


      return this.props.children(_objectSpread({}, this.props.props));
    }
  }, {
    key: "resolve",
    value: function resolve(state) {
      var _this3 = this;

      var props = this.props.props;
      var pending = Object.keys(state.pending).map(function (name) {
        var func = state.pending[name];
        return {
          name: name,
          func: func
        };
      });
      var promises = pending.map(function (_ref) {
        var func = _ref.func;
        return func(props);
      });
      var resolving = Promise.all(promises).then(function (values) {
        return values.reduce(function (resolved, value, i) {
          resolved[pending[i].name] = true;
          return resolved;
        }, {});
      }); // Resolve listeners get the current resolved

      resolving = this.onResolve(resolving); // Update current component (on client)

      resolving.then(function (resolved) {
        _this3[HAS_RESOLVED] = true;

        if (!_this3[IS_CLIENT]) {
          return false;
        }

        _this3.setAtomicState({
          pending: {},
          resolved: _objectSpread({}, state.resolved, resolved)
        });
      });
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps, nextState) {
      // Prevent updating when parent is changing values
      if (this.isParentPending()) {
        return false;
      } // Prevent rendering until pending values are resolved


      if (this.isPending(nextState)) {
        return false;
      } // Update if we have resolved successfully


      return this[HAS_RESOLVED];
    }
  }, {
    key: "setAtomicState",
    value: function setAtomicState(nextState) {
      if (this.unMounted) {
        return;
      }

      this.setState(nextState);
    }
  }]);

  return Resolver;
}(React.Component);

_defineProperty(Resolver, "childContextTypes", {
  resolver: PropTypes.object
});

_defineProperty(Resolver, "contextTypes", {
  resolver: PropTypes.object
});

_defineProperty(Resolver, "defaultProps", {
  props: {},
  resolve: {},
  cache: true,
  cacheValidator: null
});

_defineProperty(Resolver, "displayName", 'Resolver');

_defineProperty(Resolver, "propTypes", {
  children: PropTypes.func.isRequired,
  props: PropTypes.object,
  resolve: PropTypes.object,
  cache: PropTypes.bool,
  cacheValidator: PropTypes.func
});

_defineProperty(Resolver, "renderClient", function (render, node) {
  ReactDOM.hydrate(React.createElement(Resolver, null, render), node);
});

_defineProperty(Resolver, "renderServer", function (_render) {
  var initialData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var queue = [];
  server.renderToStaticMarkup(React.createElement(Resolver, {
    onResolve: function onResolve(promise) {
      queue.push(promise);
      return Promise.resolve(true);
    }
  }, _render));
  return Promise.all(queue).then(function (results) {
    var formatResults = {};
    results.forEach(function (item) {
      return Object.keys(item).forEach(function (key) {
        formatResults[key] = item[key];
      });
    });

    var data = _objectSpread({}, initialData, formatResults);

    if (Object.keys(initialData).length < Object.keys(data).length) {
      return Resolver.renderServer(_render, data);
    }

    var Resolved =
    /*#__PURE__*/
    function (_React$Component2) {
      _inherits(Resolved, _React$Component2);

      function Resolved() {
        _classCallCheck(this, Resolved);

        return _possibleConstructorReturn(this, _getPrototypeOf(Resolved).apply(this, arguments));
      }

      _createClass(Resolved, [{
        key: "render",
        value: function render() {
          return React.createElement(Resolver, null, _render);
        }
      }]);

      return Resolved;
    }(React.Component);

    _defineProperty(Resolved, "displayName", 'Resolved');

    return Resolved;
  });
});

var resolve = (function (prop, promise) {
  var cache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var cacheValid = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
  var asyncProps = _typeof(prop) === 'object' ? prop : _defineProperty({}, prop, promise);
  var asyncNames = Object.keys(asyncProps).map(capitalize).join('');
  var cacheEnabled = _typeof(prop) === 'object' && typeof promise === 'boolean' ? promise : typeof cache === 'function' ? true : cache;
  var cacheValidator = typeof cache === 'function' ? cache : cacheValid;
  return function (Component) {
    var _class, _temp;

    return _temp = _class =
    /*#__PURE__*/
    function (_React$Component) {
      _inherits(_class, _React$Component);

      function _class() {
        _classCallCheck(this, _class);

        return _possibleConstructorReturn(this, _getPrototypeOf(_class).apply(this, arguments));
      }

      _createClass(_class, [{
        key: "render",
        value: function render() {
          return React.createElement(Resolver, {
            props: this.props,
            resolve: asyncProps,
            cache: cacheEnabled,
            cacheValidator: cacheValidator
          }, function (resolved) {
            return React.createElement(Component, resolved);
          });
        }
      }]);

      return _class;
    }(React.Component), _defineProperty(_class, "displayName", "".concat(asyncNames, "Resolver")), _temp;
  };
});

var client = (function (prop, Loader) {
  var promise = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var loadProps = typeof prop === 'string' ? [prop] : prop;
  var names = loadProps.map(capitalize).join('');
  return function (Component) {
    var _class, _temp;

    return _temp = _class =
    /*#__PURE__*/
    function (_React$Component) {
      _inherits(_class, _React$Component);

      function _class(props, context) {
        var _this;

        _classCallCheck(this, _class);

        _this = _possibleConstructorReturn(this, _getPrototypeOf(_class).call(this, props, context));

        _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "enqueue", function (promise) {
          _this.queue.push(promise);

          return promise;
        });

        _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "isLoaded", function () {
          var loaded = true;
          loadProps.forEach(function (prop) {
            if (!hasOwnProperty(_this.props, prop) || !_this.props[prop]) {
              loaded = false;
            }
          });
          return loaded;
        });

        _this.queue = [];
        _this.state = {
          bypass: !canUseDom() && !promise || process.env.NODE_ENV === 'test',
          loaded: _this.isLoaded(props)
        };
        return _this;
      }

      _createClass(_class, [{
        key: "componentDidMount",
        value: function componentDidMount() {
          if (promise) {
            promise(this.props);
          }
        }
      }, {
        key: "render",
        value: function render() {
          var _this2 = this;

          var bypass = this.state.bypass;

          if (bypass || this.isLoaded()) {
            return React.createElement(Component, this.props);
          }

          return React.createElement(React.Fragment, null, React.createElement(Loader, null), !promise && React.createElement("div", {
            style: {
              display: 'none'
            }
          }, React.createElement(Resolver, {
            onResolve: this.enqueue
          }, function (resolved) {
            return React.createElement(Component, _extends({}, _this2.props, resolved));
          })));
        }
      }]);

      return _class;
    }(React.Component), _defineProperty(_class, "displayName", "".concat(names, "ClientResolver")), _defineProperty(_class, "contextTypes", {
      resolver: PropTypes.object
    }), _temp;
  };
});

exports.resolve = resolve;
exports.client = client;
exports.Resolver = Resolver;
//# sourceMappingURL=react-esc-resolver.development.js.map
